﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Net;
using System.Threading;
using Microsoft.WindowsAzure;
using Microsoft.WindowsAzure.Diagnostics;
using Microsoft.WindowsAzure.ServiceRuntime;
using Microsoft.WindowsAzure.StorageClient;
using nClam;
using System.IO;

namespace WorkerRole
{
    public class WorkerRole : RoleEntryPoint
    {
        private void FreshClam(bool notify)
        {
            var clamPath = RoleEnvironment.GetLocalResource("clamav").RootPath;
            var args = string.Format("--datadir={0}", Path.Combine(clamPath, "db"));
            if (notify)
            {
                args += string.Format(" --daemon-notify={0}", Path.Combine(clamPath, "clamd.conf"));
            }
            var proc = Process.Start(Environment.ExpandEnvironmentVariables(@"%RoleRoot%\approot\clamav\freshclam.exe"), args);
            proc.WaitForExit();
            if (proc.ExitCode != 0)
            {
                throw new Exception("freshclam failed");
            }
        }

        public override void Run()
        {
            var account = CloudStorageAccount.Parse(RoleEnvironment.GetConfigurationSettingValue("DataConnectionString"));
            var blobs = account.CreateCloudBlobClient();
            var incomingContainer = blobs.GetContainerReference("incoming");
            var publicContainer = blobs.GetContainerReference("public");
            var quarantineContainer = blobs.GetContainerReference("quarantined");
            var q = account.CreateCloudQueueClient().GetQueueReference("incoming");
            
            DateTime lastRefresh = DateTime.UtcNow;

            // wait for clamd to be up
            while (true)
            {
                try
                {
                    if (new ClamClient("127.0.0.1", 3310).Ping()) break;
                } catch { }
                Thread.Sleep(TimeSpan.FromSeconds(5));
            }

            while (true)
            {
                if (DateTime.UtcNow - lastRefresh > TimeSpan.FromMinutes(15))
                {
                    FreshClam(true);
                    lastRefresh = DateTime.UtcNow;
                }
                var msg = q.GetMessage();
                if (msg != null)
                {
                    var name = msg.AsString;
                    var blob = incomingContainer.GetBlobReference(name);
                    var result = new ClamClient("127.0.0.1", 3310).SendAndScanFile(blob.DownloadByteArray()).Result;
                    if (result == ClamScanResults.Clean)
                    {
                        publicContainer.GetBlobReference(name).CopyFromBlob(blob);
                        blob.Delete();
                    }
                    else
                    {
                        var publicBlob = publicContainer.GetBlobReference(name);
                        publicBlob.Metadata["quarantined"] = "true";
                        publicBlob.UploadText("File quarantined: possible infection.");
                        quarantineContainer.GetBlobReference(name).CopyFromBlob(blob);
                        blob.Delete();
                    }
                    q.DeleteMessage(msg);
                }
                else
                {
                    Thread.Sleep(TimeSpan.FromSeconds(1));
                }
            }
        }

        public override bool OnStart()
        {
            ServicePointManager.DefaultConnectionLimit = 12;

            var account = CloudStorageAccount.Parse(RoleEnvironment.GetConfigurationSettingValue("DataConnectionString"));
            var blobs = account.CreateCloudBlobClient();
            blobs.GetContainerReference("incoming").CreateIfNotExist();
            blobs.GetContainerReference("quarantined").CreateIfNotExist();
            var container = blobs.GetContainerReference("public");
            if (container.CreateIfNotExist())
            {
                container.SetPermissions(new BlobContainerPermissions { PublicAccess = BlobContainerPublicAccessType.Blob });
            }
            account.CreateCloudQueueClient().GetQueueReference("incoming").CreateIfNotExist();

            var clamPath = RoleEnvironment.GetLocalResource("clamav").RootPath;
            Directory.CreateDirectory(Path.Combine(clamPath, "db"));
            File.WriteAllText(Path.Combine(clamPath, "clamd.conf"),
                string.Format("TCPSocket 3310\nMaxThreads 2\nLogFile {0}\nDatabaseDirectory {1}",
                    Path.Combine(clamPath, "clamd.log"), Path.Combine(clamPath, "db")));

            FreshClam(false);
            Process.Start(Environment.ExpandEnvironmentVariables(@"%RoleRoot%\approot\clamav\clamd.exe"),
                "-c " + Path.Combine(clamPath, "clamd.conf"));

            return base.OnStart();
        }
    }
}